@@ -4,29 +4,23 @@ from typing import TYPE_CHECKING, TypeVar
 
 from attr import define, field
 from hikari import UNDEFINED, CommandOption, Permissions, Snowflakeish
-from hikari.api import CommandBuilder, EntityFactory
+from hikari.api import EntityFactory
 
-from crescent.exceptions import HikariMoment
+from crescent.context.utils import support_custom_context
 
 if TYPE_CHECKING:
     from typing import Any, Sequence, Type
 
-    from hikari import (
-        CommandType,
-        PartialApplication,
-        PartialCommand,
-        PartialGuild,
-        Snowflake,
-        SnowflakeishOr,
-        UndefinedNoneOr,
-        UndefinedOr,
-        UndefinedType,
-    )
-    from hikari.api.rest import RESTClient
+    from hikari import CommandType, Snowflake, UndefinedNoneOr, UndefinedOr, UndefinedType
 
     from crescent.commands.groups import Group, SubGroup
-    from crescent.internal.meta_struct import MetaStruct
-    from crescent.typedefs import AutocompleteCallbackT, CommandCallbackT, HookCallbackT
+    from crescent.internal.includable import Includable
+    from crescent.typedefs import (
+        CommandCallbackT,
+        HookCallbackT,
+        TransformedAutocompleteCallbackT,
+        TransformedHookCallbackT,
+    )
 
     Self = TypeVar("Self")
 
@@ -48,13 +42,11 @@ class Unique:
             self.sub_group = None
 
     @classmethod
-    def from_meta_struct(
-        cls: Type[Unique], command: MetaStruct[CommandCallbackT, AppCommandMeta]
-    ) -> Unique:
+    def from_meta_struct(cls: Type[Unique], command: Includable[AppCommandMeta]) -> Unique:
         return cls(
-            name=command.metadata.app.name,
-            type=command.metadata.app.type,
-            guild_id=command.metadata.app.guild_id,
+            name=command.metadata.app_command.name,
+            type=command.metadata.app_command.type,
+            guild_id=command.metadata.app_command.guild_id,
             group=command.metadata.group.name if command.metadata.group else None,
             sub_group=command.metadata.sub_group.name if command.metadata.sub_group else None,
         )
@@ -62,9 +54,9 @@ class Unique:
     @classmethod
     def from_app_command_meta(cls: Type[Unique], command: AppCommandMeta) -> Unique:
         return cls(
-            name=command.app.name,
-            type=command.app.type,
-            guild_id=command.app.guild_id,
+            name=command.app_command.name,
+            type=command.app_command.type,
+            guild_id=command.app_command.guild_id,
             group=command.group.name if command.group else None,
             sub_group=command.sub_group.name if command.sub_group else None,
         )
@@ -74,7 +66,7 @@ __all__: Sequence[str] = ("AppCommandMeta", "AppCommand")
 
 
 @define
-class AppCommand(CommandBuilder):
+class AppCommand:
     """Local representation of an Application Command"""
 
     type: CommandType
@@ -136,47 +128,43 @@ class AppCommand(CommandBuilder):
 
         return out
 
-    def set_id(self, _id: UndefinedOr[Snowflakeish]) -> AppCommand:
-        if isinstance(_id, int):
-            _id = Snowflake(_id)
-        self.id = _id
-        return self
-
-    def set_is_dm_enabled(self: Self, state: UndefinedOr[bool], /) -> Self:  # noqa
-        raise HikariMoment()
-
-    async def create(  # noqa
-        self,
-        rest: RESTClient,
-        application: SnowflakeishOr[PartialApplication],
-        /,
-        *,
-        guild: UndefinedOr[SnowflakeishOr[PartialGuild]] = UNDEFINED,
-    ) -> PartialCommand:
-        raise HikariMoment()
-
-    def set_default_member_permissions(  # noqa
-        self: Self, default_member_permissions: UndefinedType | int | Permissions, /
-    ) -> Self:
-        raise HikariMoment()
-
 
 @define
 class AppCommandMeta:
-    app: AppCommand
-    autocomplete: dict[str, AutocompleteCallbackT] = field(factory=dict)
+    app_command: AppCommand
+    owner: Any
+    """The function or class that was used to create the command"""
+    callback: CommandCallbackT
+    autocomplete: dict[str, TransformedAutocompleteCallbackT] = field(factory=dict)
     group: Group | None = None
     sub_group: SubGroup | None = None
-    deprecated: bool = False
-    hooks: list[HookCallbackT] = field(factory=list)
-    after_hooks: list[HookCallbackT] = field(factory=list)
+    hooks: list[TransformedHookCallbackT] = field(factory=list)
+    after_hooks: list[TransformedHookCallbackT] = field(factory=list)
+
+    def add_hooks(
+        self, hooks: Sequence[HookCallbackT], prepend: bool = False, *, after: bool
+    ) -> None:
+        transformed_hooks: list[TransformedHookCallbackT] = [
+            support_custom_context(hook) for hook in hooks
+        ]
+
+        def extend_or_prepend(list_to_edit: list[TransformedHookCallbackT]) -> None:
+            if prepend:
+                list_to_edit[:0] = transformed_hooks
+            else:
+                list_to_edit.extend(transformed_hooks)
+
+        if not after:
+            extend_or_prepend(self.hooks)
+        else:
+            extend_or_prepend(self.after_hooks)
 
     @property
     def unique(self) -> Unique:
         return Unique(
-            self.app.name,
-            self.app.type,
-            self.app.guild_id,
+            self.app_command.name,
+            self.app_command.type,
+            self.app_command.guild_id,
             self.group.name if self.group else None,
             self.sub_group.name if self.sub_group else None,
         )
