@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from contextlib import suppress
 from logging import getLogger
-from typing import TYPE_CHECKING, Mapping, cast
+from typing import TYPE_CHECKING, NamedTuple, TypeVar, cast
 
 from hikari import (
     UNDEFINED,
@@ -14,7 +14,7 @@ from hikari import (
     Snowflake,
 )
 
-from crescent.context import Context
+from crescent.context import AutocompleteContext, Context
 from crescent.internal.app_command import Unique
 from crescent.mentionable import Mentionable
 from crescent.utils import unwrap
@@ -31,8 +31,11 @@ if TYPE_CHECKING:
     )
 
     from crescent.bot import Bot
-    from crescent.internal import AppCommandMeta, MetaStruct
-    from crescent.typedefs import CommandCallbackT, HookCallbackT, OptionTypesT
+    from crescent.context import BaseContext
+    from crescent.internal import AppCommandMeta, Includable
+    from crescent.typedefs import TransformedHookCallbackT
+
+    ContextT = TypeVar("ContextT", bound=BaseContext)
 
 
 _log = getLogger(__name__)
@@ -51,55 +54,60 @@ async def handle_resp(event: InteractionCreateEvent) -> None:
         interaction = cast(CommandInteraction, interaction)
         bot = cast(Bot, bot)
 
-    ctx = _context_from_interaction_resp(interaction)
+    command_name, group, sub_group, _ = _get_crescent_command_data(interaction)
 
     command = _get_command(
-        bot, ctx.command, int(ctx.command_type), ctx.guild_id, ctx.group, ctx.sub_group
+        bot, command_name, interaction.command_type, interaction.guild_id, group, sub_group
     )
 
     if not command:
         if not bot.allow_unknown_interactions:
             _log.warning(
-                f"Handler for command `{ctx.command}` does not exist locally. (If this is"
+                f"Handler for command `{command_name}` does not exist locally. (If this is"
                 " intended, add `allow_unknown_interactions=True` to the Bot's constructor.)"
             )
         return
 
     if interaction.type is InteractionType.AUTOCOMPLETE:
-        await _handle_autocomplete_resp(command, ctx)
+        await _handle_autocomplete_resp(
+            command, _context_from_interaction_resp(AutocompleteContext, interaction)
+        )
+
         return
 
-    await _handle_slash_resp(bot, command, ctx)
+    await _handle_slash_resp(bot, command, _context_from_interaction_resp(Context, interaction))
 
 
-async def _handle_hooks(hooks: Sequence[HookCallbackT], ctx: Context) -> bool:
+async def _handle_hooks(
+    hooks: Sequence[TransformedHookCallbackT], ctx: BaseContext
+) -> tuple[bool, BaseContext]:
     """Returns `False` if the command should not be run."""
     for hook in hooks:
-        hook_res = await hook(ctx)
+        hook_res, ctx = await hook(ctx)
 
         if hook_res and hook_res.exit:
-            return False
-    return True
+            return True, ctx
+    return False, ctx
 
 
 async def _handle_slash_resp(
-    bot: Bot, command: MetaStruct[CommandCallbackT, AppCommandMeta], ctx: Context
+    bot: Bot, command: Includable[AppCommandMeta], ctx: BaseContext
 ) -> None:
 
-    if not await _handle_hooks(command.metadata.hooks, ctx):
+    should_exit, ctx = await _handle_hooks(command.metadata.hooks, ctx)
+
+    if should_exit:
         return
 
     try:
-        await command.callback(ctx, **ctx.options)
-        await _handle_hooks(command.metadata.after_hooks, ctx)
+        await command.metadata.callback(ctx, **ctx.options)
+        _, ctx = await _handle_hooks(command.metadata.after_hooks, ctx)
     except Exception as exc:
         handled = await command.app._command_error_handler.try_handle(exc, [exc, ctx])
-        await bot.on_crescent_command_error(exc, ctx, handled)
+        await bot.on_crescent_command_error(exc, ctx.into(Context), handled)
 
 
-async def _handle_autocomplete_resp(
-    command: MetaStruct[CommandCallbackT, AppCommandMeta], ctx: Context
-) -> None:
+async def _handle_autocomplete_resp(command: Includable[AppCommandMeta], ctx: BaseContext) -> None:
     interaction = cast(AutocompleteInteraction, ctx.interaction)
 
     if not command.metadata.autocomplete:
@@ -111,10 +119,13 @@ async def _handle_autocomplete_resp(
     autocomplete = command.metadata.autocomplete[option.name]
 
     try:
-        await interaction.create_response(await autocomplete(ctx, option))
+        res, ctx = await autocomplete(ctx, option)
+        await interaction.create_response(res)
     except Exception as exc:
         handled = await command.app._autocomplete_error_handler.try_handle(exc, [exc, ctx, option])
-        await command.app.on_crescent_autocomplete_error(exc, ctx, option, handled)
+        await command.app.on_crescent_autocomplete_error(
+            exc, ctx.into(AutocompleteContext), option, handled
+        )
 
 
 def _get_option_recursive(
@@ -133,11 +144,11 @@ def _get_option_recursive(
 def _get_command(
     bot: Bot,
     name: str,
-    type: int,
+    type: CommandType | int,
     guild_id: Snowflake | None,
     group: str | None,
     sub_group: str | None,
-) -> MetaStruct[CommandCallbackT, AppCommandMeta] | None:
+) -> Includable[AppCommandMeta] | None:
 
     kwargs: dict[str, Any] = dict(name=name, type=type, group=group, sub_group=sub_group)
 
@@ -156,8 +167,17 @@ _VALUE_TYPE_LINK: dict[OptionType | int, str] = {
 }
 
 
-def _context_from_interaction_resp(interaction: CommandInteraction) -> Context:
-    name: str = interaction.command_name
+class CrescentCommandData(NamedTuple):
+    """Represents the information crescent needs to understand commands"""
+
+    command_name: str
+    group: str | None
+    sub_group: str | None
+    options: Sequence[CommandInteractionOption] | None
+
+
+def _get_crescent_command_data(interaction: CommandInteraction) -> CrescentCommandData:
+    command_name: str = interaction.command_name
     group: str | None = None
     sub_group: str | None = None
     options = interaction.options
@@ -165,22 +185,32 @@ def _context_from_interaction_resp(interaction: CommandInteraction) -> Context:
     if options:
         option = options[0]
         if option.type == 1:
-            group = name
-            name = option.name
+            group = command_name
+            command_name = option.name
             options = option.options
         elif option.type == 2:
             group = interaction.command_name
             sub_group = option.name
-            name = unwrap(option.options)[0].name
+            command_name = unwrap(option.options)[0].name
             options = unwrap(option.options)[0].options
 
-    callback_options: Mapping[str, OptionTypesT | Message | User]
+    return CrescentCommandData(
+        command_name=command_name, group=group, sub_group=sub_group, options=options
+    )
+
+
+def _context_from_interaction_resp(
+    context_t: type[ContextT], interaction: CommandInteraction
+) -> ContextT:
+
+    command_name, group, sub_group, options = _get_crescent_command_data(interaction)
+
     if interaction.command_type is CommandType.SLASH:
         callback_options = _options_to_kwargs(interaction, options)
     else:
         callback_options = _resolved_data_to_kwargs(interaction)
 
-    return Context(
+    return context_t(
         interaction=interaction,
         app=cast("Bot", interaction.app),
         application_id=interaction.application_id,
@@ -192,11 +222,14 @@ def _context_from_interaction_resp(interaction: CommandInteraction) -> Context:
         guild_id=interaction.guild_id,
         user=interaction.user,
         member=interaction.member,
-        command=name,
+        command=command_name,
         group=group,
         sub_group=sub_group,
         command_type=CommandType(interaction.command_type),
         options=callback_options,
+        # See crescent/context/base_context.py
+        has_created_message=False,  # pyright: ignore
+        has_deferred_response=False,  # pyright: ignore
     )
 
 
