@@ -16,11 +16,11 @@ from hikari import (
     Snowflake,
     UndefinedType,
 )
-from hikari.api import CommandBuilder
 
+from crescent.context.utils import support_custom_context
 from crescent.exceptions import AlreadyRegisteredError
 from crescent.internal.app_command import AppCommand, AppCommandMeta, Unique
-from crescent.internal.meta_struct import MetaStruct
+from crescent.internal.includable import Includable
 from crescent.utils import gather_iter, unwrap
 
 if TYPE_CHECKING:
@@ -29,24 +29,24 @@ if TYPE_CHECKING:
     from hikari import Snowflakeish
 
     from crescent.bot import Bot
-    from crescent.typedefs import AutocompleteCallbackT
+    from crescent.typedefs import AutocompleteCallbackT, CanBuild, CommandCallbackT
 
     T = TypeVar("T", bound="Callable[..., Awaitable[Any]]")
 
-
 _log = getLogger(__name__)
 
 
-def _plugin_unload_callback(self: MetaStruct[Any, Any]) -> None:
+def _plugin_unload_callback(self: Includable[Any]) -> None:
     self.app._command_handler.remove(self)
 
 
-def _command_app_set_hook(self: MetaStruct[T, AppCommandMeta]) -> None:
+def _command_app_set_hook(self: Includable[AppCommandMeta]) -> None:
     self.app._command_handler.register(self)
 
 
 def register_command(
-    callback: T,
+    owner: Any,
+    callback: CommandCallbackT,
     command_type: CommandType,
     name: str,
     guild: Snowflakeish | None = None,
@@ -54,21 +54,20 @@ def register_command(
     options: Sequence[CommandOption] | None = None,
     default_member_permissions: UndefinedType | int | Permissions = UNDEFINED,
     dm_enabled: bool = True,
-    deprecated: bool = False,
     autocomplete: dict[str, AutocompleteCallbackT] = {},
-) -> MetaStruct[T, AppCommandMeta]:
+) -> Includable[AppCommandMeta]:
 
     if not iscoroutinefunction(callback):
         raise ValueError(f"`{callback.__name__}` must be an async function.")
 
-    meta: MetaStruct[T, AppCommandMeta] = MetaStruct(
-        callback=callback,
+    includable: Includable[AppCommandMeta] = Includable(
         app_set_hooks=[_command_app_set_hook],
         plugin_unload_hooks=[_plugin_unload_callback],
         metadata=AppCommandMeta(
-            deprecated=deprecated,
-            autocomplete=autocomplete,
-            app=AppCommand(
+            owner=owner,
+            callback=support_custom_context(callback),
+            autocomplete={k: support_custom_context(v) for k, v in autocomplete.items()},
+            app_command=AppCommand(
                 type=command_type,
                 description=description,
                 guild_id=guild,
@@ -80,28 +79,28 @@ def register_command(
         ),
     )
 
-    return meta
+    return includable
 
 
 _E = TypeVar("_E", bound="Callable[..., Awaitable[Any]]")
 
 
 class ErrorHandler(Generic[_E]):
-    __slots__: Sequence[str] = ("bot", "registry")
+    __slots__: Sequence[str] = ("bot", "registry", "supports_custom_ctx")
 
     def __init__(self, bot: Bot):
         self.bot: Bot = bot
-        self.registry: dict[type[Exception], MetaStruct[_E, Any]] = {}
+        self.registry: dict[type[Exception], Includable[_E]] = {}
 
-    def register(self, meta: MetaStruct[_E, Any], exc: type[Exception]) -> None:
-        if reg_meta := self.registry.get(exc):
+    def register(self, includable: Includable[_E], exc: type[Exception]) -> None:
+        if reg_includable := self.registry.get(exc):
             raise AlreadyRegisteredError(
-                f"`{getattr(meta.callback, '__name__')}` can not catch `{exc.__name__}`."
+                f"`{getattr(includable.metadata, '__name__')}` can not catch `{exc.__name__}`."
                 f" `{exc.__name__}` is already registered to"
-                f" `{reg_meta.callback.__name__}`."
+                f" `{reg_includable.metadata.__name__}`."
             )
 
-        self.registry[exc] = meta
+        self.registry[exc] = includable
 
     def remove(self, exc: type[Exception]) -> None:
         self.registry.pop(exc)
@@ -112,7 +111,7 @@ class ErrorHandler(Generic[_E]):
         was handled.
         """
         if func := self.registry.get(exc.__class__):
-            await func.callback(*args)
+            await func.metadata(*args)
             return True
 
         return False
@@ -127,19 +126,16 @@ class CommandHandler:
         self.guilds: Sequence[Snowflakeish] = guilds
         self.application_id: Snowflake | None = None
 
-        self.registry: dict[
-            Unique, MetaStruct["Callable[..., Awaitable[Any]]", AppCommandMeta]
-        ] = {}
+        self.registry: dict[Unique, Includable[AppCommandMeta]] = {}
 
-    def register(self, command: MetaStruct[T, AppCommandMeta]) -> MetaStruct[T, AppCommandMeta]:
-        command.metadata.app.guild_id = command.metadata.app.guild_id or self.bot.default_guild
-        # NOTE: T is bound to Callable[..., Awaitable[Any]], so we can cast it safely. Mypy's
-        # support for TypeVars is bad, so it doesn't understand this.
-        _command = cast("MetaStruct[Callable[..., Awaitable[Any]], AppCommandMeta]", command)
-        self.registry[command.metadata.unique] = _command
+    def register(self, command: Includable[AppCommandMeta]) -> Includable[AppCommandMeta]:
+        command.metadata.app_command.guild_id = (
+            command.metadata.app_command.guild_id or self.bot.default_guild
+        )
+        self.registry[command.metadata.unique] = command
         return command
 
-    def remove(self, command: MetaStruct[T, AppCommandMeta]) -> None:
+    def remove(self, command: Includable[AppCommandMeta]) -> None:
         self.registry.pop(command.metadata.unique)
 
     def build_commands(self) -> Sequence[AppCommand]:
@@ -147,10 +143,7 @@ class CommandHandler:
         built_commands: dict[Unique, AppCommand] = {}
 
         for command in self.registry.values():
-            if command.metadata.deprecated:
-                continue
-
-            elif command.metadata.sub_group:
+            if command.metadata.sub_group:
                 # If a command has a sub_group, it must be nested 2 levels deep.
                 #
                 # command
@@ -168,8 +161,8 @@ class CommandHandler:
                 # hold the subcommand.
                 key = Unique(
                     name=unwrap(command.metadata.group).name,
-                    type=command.metadata.app.type,
-                    guild_id=command.metadata.app.guild_id,
+                    type=command.metadata.app_command.type,
+                    guild_id=command.metadata.app_command.guild_id,
                     group=None,
                     sub_group=None,
                 )
@@ -178,11 +171,13 @@ class CommandHandler:
                     built_commands[key] = AppCommand(
                         name=unwrap(command.metadata.group).name,
                         description=unwrap(command.metadata.group).description or "No Description",
-                        type=command.metadata.app.type,
-                        guild_id=command.metadata.app.guild_id,
+                        type=command.metadata.app_command.type,
+                        guild_id=command.metadata.app_command.guild_id,
                         options=[],
-                        default_member_permissions=command.metadata.app.default_member_permissions,
-                        is_dm_enabled=command.metadata.app.is_dm_enabled,
+                        default_member_permissions=(
+                            command.metadata.app_command.default_member_permissions
+                        ),
+                        is_dm_enabled=command.metadata.app_command.is_dm_enabled,
                     )
 
                 # The top-level command now exists. A subcommand group now if placed
@@ -213,10 +208,10 @@ class CommandHandler:
 
                 cast("list[CommandOption]", sub_command_group.options).append(
                     CommandOption(
-                        name=command.metadata.app.name,
-                        description=unwrap(command.metadata.app.description),
+                        name=command.metadata.app_command.name,
+                        description=unwrap(command.metadata.app_command.description),
                         type=OptionType.SUB_COMMAND,
-                        options=command.metadata.app.options,
+                        options=command.metadata.app_command.options,
                         is_required=False,
                     )
                 )
@@ -234,8 +229,8 @@ class CommandHandler:
                 # hold the subcommand.
                 key = Unique(
                     name=command.metadata.group.name,
-                    type=command.metadata.app.type,
-                    guild_id=command.metadata.app.guild_id,
+                    type=command.metadata.app_command.type,
+                    guild_id=command.metadata.app_command.guild_id,
                     group=None,
                     sub_group=None,
                 )
@@ -244,38 +239,41 @@ class CommandHandler:
                     built_commands[key] = AppCommand(
                         name=command.metadata.group.name,
                         description=unwrap(command.metadata.group).description or "No Description",
-                        type=command.metadata.app.type,
-                        guild_id=command.metadata.app.guild_id,
+                        type=command.metadata.app_command.type,
+                        guild_id=command.metadata.app_command.guild_id,
                         options=[],
-                        default_member_permissions=command.metadata.app.default_member_permissions,
-                        is_dm_enabled=command.metadata.app.is_dm_enabled,
+                        default_member_permissions=(
+                            command.metadata.app_command.default_member_permissions
+                        ),
+                        is_dm_enabled=command.metadata.app_command.is_dm_enabled,
                     )
 
                 # No checking has to be done before appending `command` since it is the
                 # lowest level.
                 cast("list[CommandOption]", built_commands[key].options).append(
                     CommandOption(
-                        name=command.metadata.app.name,
-                        description=unwrap(command.metadata.app.description),
-                        type=command.metadata.app.type,
-                        options=command.metadata.app.options,
+                        name=command.metadata.app_command.name,
+                        description=unwrap(command.metadata.app_command.description),
+                        type=command.metadata.app_command.type,
+                        options=command.metadata.app_command.options,
                         is_required=False,
                     )
                 )
 
             else:
-                built_commands[Unique.from_meta_struct(command)] = command.metadata.app
+                built_commands[Unique.from_meta_struct(command)] = command.metadata.app_command
 
         return tuple(built_commands.values())
 
-    async def post_guild_commands(
-        self, commands: Sequence[CommandBuilder], guild: Snowflakeish
-    ) -> None:
+    async def post_guild_commands(self, commands: Sequence[CanBuild], guild: Snowflakeish) -> None:
         try:
             if self.application_id is None:
                 raise AttributeError("Client `application_id` is not defined")
             await self.bot.rest.set_application_commands(
-                application=self.application_id, commands=commands, guild=guild
+                application=self.application_id,
+                # The only method that is called has been implemented.
+                commands=commands,  # type: ignore
+                guild=guild,
             )
         except ForbiddenError:
             if guild in self.bot.cache.get_guilds_view().keys():
@@ -310,7 +308,9 @@ class CommandHandler:
         assert self.application_id is not None
         await gather(
             self.bot.rest.set_application_commands(
-                application=self.application_id, commands=global_commands
+                application=self.application_id,
+                # The only method that is called has been implemented.
+                commands=global_commands,  # type: ignore
             ),
             gather_iter(
                 self.post_guild_commands(commands, guild)
